/*
** Copyright (C) 1995  Jonathan Paul Griffiths.  All rights reserved.
**
** You may do anything with this code you wish EXCEPT sell it. You may sell
** any software you create using this code,  but you MUST NOT charge for
** the code itself.  Charging a distribution fee for this code is also
** FORBIDDEN.
*/

/*
 these are the copying operations available. All take len in bytes.

   FAST_SHORT_COPY - uses rep movsb, good for short copies (ie < 32 bytes).
   FAST_LONG_COPY  - calculates alignment and does fastest copy -> movsw.
   WORD_COPY       - uses rep movsw, use only if src,dst and len are even.
   BYTE_COPY       - uses rep movsb, handles overlap in DJGPP.
   this one is not implemented yet!
   FAST__VERY_LONG_COPY  - calculates alignment and does fastest copy -> movsd.
*/

#define _bcopy(s,d,l)      _copy_sb((unsigned int)(s),(unsigned int)(d),(unsigned int)(l))

#define _bytecopy(s,d,l)      _blast((s),(d),(unsigned int)(l))
#define _wordcopy(s,d,l)      _blast((s),(d),(unsigned int)(len))

#define FAST_SHORT_COPY(a,b,len) _copy_sb((unsigned int)(a),(unsigned int)(b),(unsigned int)(len))
#define FAST_LONG_COPY(a,b,len)  _blast((a),(b),(unsigned int)(len))
#define WORD_COPY(a,b,len)       _copy_sw((unsigned int)(a),(unsigned int)(b),(unsigned int)(len))
#define BYTE_COPY(a,b,len)       _bcopy((a),(b),(len))

/* all of these copies are source, dest, len */

/* copy methods */
#define USE_SB         0 /* use rep movsb to copy */
#define INC_USE_SW     1 /* use one rep movsb then use rep movsw to copy */
#define USE_SW         2 /* use rep movsw to copy */
#define USE_SW_INC     3 /* use rep movsw to copy then use one rep movsb */
#define USE_INC_SW_INC 4 /* use one rep movsb then use rep movsw then use one rep movsb */

/* return the best copy method for two addresses and a length */
int get_copy_method2(unsigned int addr1,unsigned int addr2,unsigned int len);

/* other copying routines */
void _blast(void *src,void *dest,unsigned int len);


/* ========================================================================**
** Try to ignore this mess,  I had to this because the compiler wouldn't   **
** inline them.                                                            **
** ========================================================================*/

/* all of these copies are source, dest, len */
#define _copy_sb(src,dest,len) asm volatile(                              "\n\
	push    %%ecx                                                     \n\
	movl    %0,%%esi                                                  \n\
	movl    %1,%%edi                                                  \n\
	movl	%2,%%ecx						  \n\
	rep                                                               \n\
	movsb                                                             \n\
	pop %%ecx                                                          "\
	: /* NOTHING */                                                    \
	: "g" (src), "g" (dest), "g" (len)                                 \
	: "di", "si", "cx" )

#define _copy_sw(src,dest,len) asm volatile(                              "\n\
	push    %%ecx                                                     \n\
	movl    %0,%%esi                                                  \n\
	movl    %1,%%edi                                                  \n\
	movl    %2,%%ecx                                                  \n\
	shrl    $1,%%ecx                                                  \n\
	rep                                                               \n\
	movsw                                                             \n\
	pop %%ecx                                                         "\
	: /* NOTHING */                                                    \
	: "g" (src), "g" (dest), "g" (len)                                 \
	: "di", "si", "cx" )

#define _copy_sw_inc(src,dest,len) asm volatile(                         "\n\
	push    %%ecx                                                     \n\
	movl    %0,%%esi                                                  \n\
	movl    %1,%%edi                                                  \n\
	movl    %2,%%ecx                                                  \n\
	shrl    $1,%%ecx                                                  \n\
	rep                                                               \n\
	movsw                                                             \n\
	movl    $1,%%ecx                                                  \n\
	rep                                                               \n\
	movsb                                                             \n\
	pop     %%ecx                                                     "\
	: /* NOTHING */                                                    \
	: "g" (src), "g" (dest), "g" (len)                                 \
	: "di", "si", "cx" )

#define _copy_inc_sw(src,dest,len) asm volatile(                         "\n\
	push    %%ecx                                                     \n\
	movl    %0,%%esi                                                  \n\
	movl    %1,%%edi                                                  \n\
	movl    $1,%%ecx                                                  \n\
	rep                                                               \n\
	movsb                                                             \n\
	movl    %2,%%ecx                                                  \n\
	shrl    $1,%%ecx                                                  \n\
	rep                                                               \n\
	movsw                                                             \n\
	pop     %%ecx                                                    "\
	: /* NOTHING */                                                    \
	: "g" (src), "g" (dest), "g" (len)                                 \
	: "di", "si", "cx" )

#define _copy_inc_sw_inc(src,dest,len) asm volatile(                     "\n\
	push    %%ecx                                                     \n\
	movl    %0,%%esi                                                  \n\
	movl    %1,%%edi                                                  \n\
	movl    $1,%%ecx                                                  \n\
	rep                                                               \n\
	movsb                                                             \n\
	movl    %2,%%ecx                                                  \n\
	shrl    $1,%%ecx                                                  \n\
	rep                                                               \n\
	movsw                                                             \n\
	movl    $1,%%ecx                                                  \n\
	rep                                                               \n\
	movsb                                                             \n\
	pop     %%ecx                                                     "\
	: /* NOTHING */                                                    \
	: "g" (src), "g" (dest), "g" (len)                                 \
	: "di", "si", "cx" )

