/*
	this is the definition file for the
	inline DIB macros functions specified withing
	dibstuff.h

	Mark Gross June 3, 1995
*/

#include <windows.h>
#include <windowsx.h>
#include "dibstuff.h"

inline HANDLE HandleFromDib(LPBITMAPINFOHEADER lpbi)
{
#ifdef  WIN32
	 return  GlobalHandle(lpbi);
#else
	 return (HANDLE)GlobalHandle(SELECTOROF(lpbi));
#endif
}

inline PDIB DibFromHandle(HGLOBAL h)
{
	return (PDIB)GlobalLock(h);
}

inline void DibFree(PDIB pdib)
{
	GlobalFreePtr(pdib);
}

inline unsigned long WIDTHBYTES(long cbits)
{
	return 	((((unsigned long)cbits + 31UL)&(~31UL))/8UL);  /* DWORD aligned ! */
}

inline long DibWidth(PDIB pdib)
{
	return  pdib->biWidth;
}

inline long DibHeight(PDIB pdib)
{
	if (pdib->biHeight < 0)
		return -(pdib->biHeight);
	else
		return pdib->biHeight;
}

inline unsigned short DibBitCount(PDIB pdib)
{
	return  pdib->biBitCount;
}


inline unsigned long DibCompression(PDIB pdib)
{
	return  pdib->biCompression;
}

inline long DibWidthBytesN(PDIB pdib, long n)
{
	return WIDTHBYTES(DibWidth(pdib) * n);
}

inline long DibWidthBytes(PDIB pdib)
{
	return DibWidthBytesN(pdib, DibBitCount(pdib));
}

inline long DibPitch(PDIB pdib)
{
	return DibWidthBytesN(pdib, pdib->biBitCount);
}

inline unsigned long DibSizeImage(PDIB pdib)
{
	 if (pdib->biSizeImage == 0)
		return (DWORD)DibPitch(pdib) * (DWORD)DibHeight(pdib);
	 else
		return pdib->biSizeImage;

}

inline unsigned long DibSizeColors(PDIB pdib)
{
	return (DibNumColors(pdib) * sizeof(RGBQUAD));
}

inline unsigned long DibPaletteSize(LPBITMAPINFOHEADER lpbi)
{
	return  (DibNumColors(lpbi) * sizeof(RGBQUAD));
}

inline unsigned long DibSizeHeader(PDIB pdib)
{
	return pdib->biSize;
}

inline unsigned long DibSize(PDIB pdib)
{
	return pdib->biSize + DibSizeImage(pdib) + DibSizeColors(pdib);
}

inline short DibFlipY(LPBITMAPINFOHEADER lpbi, short y)
{
	 return (short)(lpbi->biHeight-1-y);
}

inline LPVOID DibPtr(LPBITMAPINFOHEADER lpbi)
{
//HACK for NT BI_BITFIELDS DIBs
#ifdef WIN32
	 if(lpbi->biCompression == BI_BITFIELDS)
		return (LPVOID)(DibColors(lpbi) + 3);
	 else
		return (LPVOID)(DibColors(lpbi) + (UINT)(lpbi)->biClrUsed));
#else
	 return (LPVOID)(DibColors(lpbi) + (UINT)(lpbi)->biClrUsed);
#endif

}

inline RGBQUAD FAR *DibColors(LPBITMAPINFOHEADER lpbi)
{
	return (RGBQUAD FAR *)((LPBYTE)lpbi + (int)(lpbi->biSize));
}

inline unsigned long DibNumColors(LPBITMAPINFOHEADER lpbi)
{
	if ((lpbi)->biClrUsed == 0 && (lpbi)->biBitCount <= 8 )
		return 1UL << lpbi->biBitCount;
	else
		return lpbi->biClrUsed;
}

inline LPVOID DibXYN(LPBITMAPINFOHEADER lpbi,HPBYTE pb,unsigned short x,unsigned short y,unsigned short n)
{
	return (LPVOID)( (BYTE _huge *)(pb) +
		(UINT)(x * n / 8U) +
			(DibWidthBytesN(lpbi,n) * (DWORD)y));
}


inline LPVOID DibXY(LPBITMAPINFOHEADER lpbi,unsigned short x,unsigned short y)
{
	return DibXYN(lpbi,(HPBYTE)DibPtr(lpbi),x,y,(lpbi)->biBitCount);
}

inline void FixBitmapInfo(LPBITMAPINFOHEADER lpbi)
{
	if ((lpbi)->biSizeImage == 0)
		(lpbi)->biSizeImage = DibSizeImage(lpbi);
	if ((lpbi)->biClrUsed == 0)
		(lpbi)->biClrUsed = DibNumColors(lpbi);
	if ((lpbi)->biCompression == BI_BITFIELDS && (lpbi)->biClrUsed == 0)
		; // (lpbi)->biClrUsed = 3;
}

inline BITMAPINFO FAR * DibInfo(PDIB pDIB)
{
	return 	((BITMAPINFO FAR *)pDIB);

}

inline long DibStorageWidth(PDIB pdib)
{
	return ((DibWidth(pdib) + 3L) & ~3L);
}

inline LPSTR DibPBits(PDIB pdib)
{
	return (LPSTR)(pdib) +  (sizeof(BITMAPINFOHEADER)
									 + (short)DibNumColors(pdib) * sizeof(RGBQUAD));
}

inline LPRGBQUAD DibPCLRTAB(PDIB pdib)
{
	return (LPRGBQUAD)((LPSTR)(pdib) + sizeof(BITMAPINFOHEADER));
}


