//****************************************************************************
// File:
//
//     DLGBOX1.CPP
//
// Purpose:
//
//     Implementation file for class CDialogBox1.
//
// Functions:
//
//     CDialogBox1::CDialogBox1    - Constructor for the CDialogBox1 class.
//     CDialogBox1::DoDataExchange - Exchanges data between class members and
//                                     dialog box controls. 
//     CDialogBox1::OnInitDialog   - Handles initial setup of dialog box.
//     CDialogBox1::OnCommandHelp  - REMOVED (not used in 32-bit sample)
//     CDialogBox1::OnHelpInfo     - Processes the WM_HELP message.
//     CDialogBox1::OnContextMenu  - Process the WM_CONTEXTMENU message used
//                                     for "What's This?" menu.
//
// Development Team:
//
//     Joel Krist
//
//     Modified:
//     James Rhodes - added support for new Win32 WM_HELP & WM_CONTEXTMENU
//                    messages.  Removed 16-bit specific code no longer needed
//
// Written by Microsoft Product Support Services, Languages Developer Support
// Copyright (c) 1993, 1995 Microsoft Corporation. All rights reserved.
//****************************************************************************

#include "stdafx.h"
#include <afxpriv.h>
#include "dlghlp32.h"
#include "dlgbox1.h"
#include "dlgres.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

//***********************************************************************
// Function:
//
//     CDialogBox1::CDialogBox1()
//
// Purpose:
//
//     Constructs a CDialogBox1 object
//
// History:
//
//   Date   Comment                                           Initials
// ======== ================================================= ========
// 11/22/93 Created                                             JKK
//***********************************************************************

CDialogBox1::CDialogBox1(CWnd* pParent /*=NULL*/)
    : CDialog(CDialogBox1::IDD, pParent)
{
    //{{AFX_DATA_INIT(CDialogBox1)
        // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
}

//***********************************************************************
// Function:
//
//     CDialogBox1::DoDataExchange()
//
// Purpose:
//
//     Handles the exchange of data between CDialogBox1 members and
//     dialog controls. Not used in this sample.
//
// History:
//
//   Date   Comment                                           Initials
// ======== ================================================= ========
// 11/22/93 Created                                             JKK
//***********************************************************************

void CDialogBox1::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CDialogBox1)
        // NOTE: the ClassWizard will add DDX and DDV calls here
    //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CDialogBox1, CDialog)
    //{{AFX_MSG_MAP(CDialogBox1)
	//}}AFX_MSG_MAP

    ON_MESSAGE( WM_COMMANDHELP, OnCommandHelp )

	// Windows 95 style popup help if using MFC 4.0
#if (_MFC_VER >= 0x400)
	ON_WM_HELPINFO()
	ON_WM_CONTEXTMENU()
#endif

END_MESSAGE_MAP()

//***********************************************************************
// Function:
//
//     CDialogBox1::OnInitDialog()
//
// Purpose:
//
//     Centers the dialog box over the application main window.
//
// History:
//
//   Date   Comment                                           Initials
// ======== ================================================= ========
// 11/22/93 Created                                             JKK
// 11/19/95 Added ModifyStyleEx for context help support		JMR
//***********************************************************************

BOOL CDialogBox1::OnInitDialog()
{
    CDialog::OnInitDialog();
    
    CenterWindow();
	ModifyStyleEx( 0, WS_EX_CONTEXTHELP );
    
    return TRUE;  // return TRUE  unless you set the focus to a control
}

// maps control id's to help contexts
DWORD CDialogBox1::m_dwHelpMap[] =
{
        IDC_RADIO1,		HID_DIALOGBOX1_RADIO1,
        IDC_RADIO2,		HID_DIALOGBOX1_RADIO2,
        IDC_RADIO3,		HID_DIALOGBOX1_RADIO3,
        IDC_BUTTON1,	HID_DIALOGBOX1_BUTTON1,
        IDC_BUTTON2,	HID_DIALOGBOX1_BUTTON2,
        IDC_EDIT1,		HID_DIALOGBOX1_EDIT1,
        IDC_EDIT2,		HID_DIALOGBOX1_EDIT2,
        IDCANCEL,		HID_DIALOGBOX1_CANCEL,
        IDOK,			HID_DIALOGBOX1_OK,
		0,0
};


//***********************************************************************
// Function:
//
//     CDialogBox1::OnHelpInfo()
//
// Purpose:
//
//     Win32 sends a WM_HELP when the user presses F1 or uses the
//     context help button.  This function processes the message to
//     display popup help on the control clicked.
//
// Parameters:
//
//     pHelpInfo  The HELPINFO structure.  iContextType should be
//                HELPINFO_WINDOW when a control is clicked.
//
// Returns:
//
//     Should always return TRUE.
//
// History:
//
//   Date   Comment                                           Initials
// ======== ================================================= ========
// 11/19/95 Created                                             JMR
//***********************************************************************

BOOL CDialogBox1::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	if (pHelpInfo->iContextType == HELPINFO_WINDOW)
	{
		::WinHelp ((HWND)pHelpInfo->hItemHandle,
			AfxGetApp()->m_pszHelpFilePath,
			HELP_WM_HELP,
			(DWORD)(LPVOID)m_dwHelpMap);
	}
	
	return TRUE;
}

//***********************************************************************
// Function:
//
//     CDialogBox1::OnContextMenu()
//
// Purpose:
//
//     Win32 sends a WM_CONTEXT when the user clicks the right mouse
//     button.  This function processes the message to display the
//     "What's This?" popup menu and displays the help topic if "What's
//     This?" is selected.
//
// Parameters:
//
//     pWnd   will be this dialog box's window pointer.
//     point  where the user clicked the right mouse button.
//
// Returns:
//
//     Should always return TRUE.
//
// History:
//
//   Date   Comment                                           Initials
// ======== ================================================= ========
// 11/19/95 Created                                             JMR
//***********************************************************************

void CDialogBox1::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	if (this == pWnd)
		return;

	::WinHelp (pWnd->m_hWnd,
		AfxGetApp()->m_pszHelpFilePath,
		HELP_CONTEXTMENU,
		(DWORD)(LPVOID)m_dwHelpMap);
}

