//****************************************************************************
// File:
//
//     DLGBOX2.CPP
//
// Purpose:
//
//     Implementation file for class CDialogBox2.
//
// Functions:
//
//     CDialogBox2::CDialogBox2    - Constructor for the CDialogBox2 class.
//     CDialogBox2::DoDataExchange - Exchanges data between class members and
//                                     dialog box controls. 
//     CDialogBox2::OnInitDialog   - Handles initial setup of dialog box.
//     CDialogBox2::OnCommandHelp  - REMOVED (not used in 32-bit sample)
//     CDialogBox1::OnHelpInfo     - Processes the WM_HELP message.
//     CDialogBox1::OnContextMenu  - Process the WM_CONTEXTMENU message used
//                                     for "What's This?" menu.
//
// Development Team:
//
//     Joel Krist
//
//     Modified:
//     James Rhodes - added support for new Win32 WM_HELP & WM_CONTEXTMENU
//                    messages.
//
// Written by Microsoft Product Support Services, Languages Developer Support
// Copyright (c) 1993, 1995 Microsoft Corporation. All rights reserved.
//****************************************************************************

#include "stdafx.h"
#include <afxpriv.h>
#include "dlghlp32.h"
#include "dlgbox2.h"
#include "dlgres.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

//***********************************************************************
// Function:
//
//     CDialogBox2::CDialogBox2()
//
// Purpose:
//
//     Constructs a CDialogBox2 object
//
// History:
//
//   Date   Comment                                           Initials
// ======== ================================================= ========
// 11/22/93 Created                                             JKK
//***********************************************************************

CDialogBox2::CDialogBox2(CWnd* pParent /*=NULL*/)
    : CDialog(CDialogBox2::IDD, pParent)
{
    //{{AFX_DATA_INIT(CDialogBox2)
        // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
}

//***********************************************************************
// Function:
//
//     CDialogBox2::DoDataExchange()
//
// Purpose:
//
//     Handles the exchange of data between CDialogBox2 members and
//     dialog controls. Not used in this sample.
//
// History:
//
//   Date   Comment                                           Initials
// ======== ================================================= ========
// 11/22/93 Created                                             JKK
//***********************************************************************

void CDialogBox2::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CDialogBox2)
        // NOTE: the ClassWizard will add DDX and DDV calls here
    //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CDialogBox2, CDialog)
    //{{AFX_MSG_MAP(CDialogBox2)
    //}}AFX_MSG_MAP

	// Windows 95 style popup help if using MFC 4.0
	ON_WM_HELPINFO()
	ON_WM_CONTEXTMENU()

END_MESSAGE_MAP()

//***********************************************************************
// Function:
//
//     CDialogBox2::OnInitDialog()
//
// Purpose:
//
//     Centers the dialog box over the application main window.
//
// History:
//
//   Date   Comment                                           Initials
// ======== ================================================= ========
// 11/22/93 Created                                             JKK
// 11/19/95 Added ModifyStyleEx for context help support		JMR
//***********************************************************************

BOOL CDialogBox2::OnInitDialog()
{
    CDialog::OnInitDialog();
    
    CenterWindow();
    
// Windows 95 style popup help if using MFC 4.0
#if (_MFC_VER >= 0x400)
	ModifyStyleEx( 0, WS_EX_CONTEXTHELP );
#endif

    return TRUE;  // return TRUE  unless you set the focus to a control
}

//***********************************************************************
// Function:
//
//     CDialogBox2::OnCommandHelp()
//
// Purpose:
//
//     Handles the WM_COMMANDHELP message generated when the user presses
//     the F1 key. Determines which dialog control has the focus and
//     calls CWinApp::WinHelp with the correct help context.
//
// Parameters:
//
//     wParam - Not used and will be zero.
//     lParam - Contains the currently available help context. lParam
//              is zero if no Help context is requested. In this case
//              we always ignore the initial value of lParam and set
//              it to the correct value based upon which control has
//              the focus.
//
// Returns:
//
//     Returns TRUE if we want to stop the routing of this command to
//     other classes because we handled it and FALSE if we want the
//     command routed on because we didn't handle it.
//
// History:
//
//   Date   Comment                                           Initials
// ======== ================================================= ========
// 11/23/93 Created                                             JKK
//***********************************************************************


// Windows 95 style popup help if using MFC 4.0
DWORD CDialogBox2::m_dwHelpMap[] =
{
        IDC_RADIO1,		HID_DIALOGBOX2_RADIO1,
        IDC_RADIO2,		HID_DIALOGBOX2_RADIO2,
        IDC_RADIO3,		HID_DIALOGBOX2_RADIO3,
		IDC_RADIO4,		HID_DIALOGBOX2_RADIO4,
		0,0
};


//***********************************************************************
// Function:
//
//     CDialogBox2::OnHelpInfo()
//
// Purpose:
//
//     Win32 sends a WM_HELP when the user presses F1 or uses the
//     context help button.  This function processes the message to
//     display popup help on the control clicked.
//
// Parameters:
//
//     pHelpInfo  The HELPINFO structure.  iContextType should be
//                HELPINFO_WINDOW when a control is clicked.
//
// Returns:
//
//     Should always return TRUE.
//
// History:
//
//   Date   Comment                                           Initials
// ======== ================================================= ========
// 11/19/95 Created                                             JMR
//***********************************************************************

BOOL CDialogBox2::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	if (pHelpInfo->iContextType == HELPINFO_WINDOW)
	{
		::WinHelp ((HWND)pHelpInfo->hItemHandle,
			AfxGetApp()->m_pszHelpFilePath,
			HELP_WM_HELP,
			(DWORD)(LPVOID)m_dwHelpMap);
	}
	
	return TRUE;
}

//***********************************************************************
// Function:
//
//     CDialogBox2::OnContextMenu()
//
// Purpose:
//
//     Win32 sends a WM_CONTEXT when the user clicks the right mouse
//     button.  This function processes the message to display the
//     "What's This?" popup menu and displays the help topic if "What's
//     This?" is selected.
//
// Parameters:
//
//     pWnd   will be this dialog box's window pointer.
//     point  where the user clicked the right mouse button.
//
// Returns:
//
//     Should always return TRUE.
//
// History:
//
//   Date   Comment                                           Initials
// ======== ================================================= ========
// 11/19/95 Created                                             JMR
//***********************************************************************

void CDialogBox2::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	if (this == pWnd)
		return;

	::WinHelp (pWnd->m_hWnd,
		AfxGetApp()->m_pszHelpFilePath,
		HELP_CONTEXTMENU,
		(DWORD)(LPVOID)m_dwHelpMap);
}

