//****************************************************************************
// File:
//
//     DLGRES.H
//
// Purpose:
//
//     Defines the help context values for the controls in Dialog Box 1 and
//     Dialog Box 2.
//
//     The ID of each dialog box is used as a base to generate a unique help
//     context value for each dialog control. This allows the control ID to
//     be used to specify the help context while allowing the same control
//     ID to be used in more than one dialog.
//
//     For example, both Dialog Box 1 and Dialog Box 2 contain a radio button
//     with an ID of IDC_RADIO1. Dialog Box 1 has an ID of IDD_DIALOGBOX1
//     and Dialog Box 2 has an ID of IDD_DIALOGBOX2, both of which are
//     defined in the header file RESOURCE.H. IDD_DIALOGBOX1 and
//     IDD_DIALOGBOX2 must differ in value by enough that adding the individual
//     control ID's to them does not generate any overlapping help context
//     values.
//
//     A base value of 0x20000 is also added the help context value to cause
//     the context value for each control to be in the range of 0x20000 -
//     0x2FFFF, the help context range for windows and dialogs. For more
//     information on help context ranges, refer to MFC Technical Note 28:
//     Context-Sensitive Help Support.
//
// Development Team:
//
//     Joel Krist
//
// Written by Microsoft Product Support Services, Languages Developer Support
// Copyright (c) 1993 Microsoft Corporation. All rights reserved.
//****************************************************************************

#include <windows.h>
#include "resource.h"

#define DIALOGBASE 0x20000

// Help context ID's for controls in Dialog Box 1
#define HID_DIALOGBOX1_RADIO1    DIALOGBASE + IDD_DIALOGBOX1 + IDC_RADIO1
#define HID_DIALOGBOX1_RADIO2    DIALOGBASE + IDD_DIALOGBOX1 + IDC_RADIO2
#define HID_DIALOGBOX1_RADIO3    DIALOGBASE + IDD_DIALOGBOX1 + IDC_RADIO3
#define HID_DIALOGBOX1_BUTTON1   DIALOGBASE + IDD_DIALOGBOX1 + IDC_BUTTON1
#define HID_DIALOGBOX1_BUTTON2   DIALOGBASE + IDD_DIALOGBOX1 + IDC_BUTTON2
#define HID_DIALOGBOX1_EDIT1     DIALOGBASE + IDD_DIALOGBOX1 + IDC_EDIT1
#define HID_DIALOGBOX1_EDIT2     DIALOGBASE + IDD_DIALOGBOX1 + IDC_EDIT2
#define HID_DIALOGBOX1_OK        DIALOGBASE + IDD_DIALOGBOX1 + IDOK
#define HID_DIALOGBOX1_CANCEL    DIALOGBASE + IDD_DIALOGBOX1 + IDCANCEL

// Help context ID's for controls in Dialog Box 2.
#define HID_DIALOGBOX2_RADIO1    DIALOGBASE + IDD_DIALOGBOX2 + IDC_RADIO1
#define HID_DIALOGBOX2_RADIO2    DIALOGBASE + IDD_DIALOGBOX2 + IDC_RADIO2                    
#define HID_DIALOGBOX2_RADIO3    DIALOGBASE + IDD_DIALOGBOX2 + IDC_RADIO3
#define HID_DIALOGBOX2_RADIO4    DIALOGBASE + IDD_DIALOGBOX2 + IDC_RADIO4
