;(c) Wally Bass, 1996.
;
;'todos' and 'fromdos' map calendar dates to/from the 'date' value
;as returned by a DOS CLOCK$ device driver
;
;This file is essentially a simplified Julian date routine which only
;covers the 'DOS era.' Because it only covers from 1/1/80 to 2/28/2100,
;operands can be kept to 16 bit size, and code pertaining to omission of
;leap years on century years not divisible by 400 can be omitted.
;
;'todos' maps valid dates onto a contiguous integer space, from 1/1/1980
;(=0) thru 2/28/2100(=43888)
;
;in:	CH: month (1-12)
;	CL: day of month (1-31)
;	AX: year (1980-2100)
;out:	CX: number of days since 1/1/80
;	DX: day of week (0-6=Sun thru Sat)
;	AX,BX: destroyed
;	SI,DI,BP,SS,DS,ES are preserved
;
todos	proc near
	sub	ax,1956
	cmp	ch,3		;check if March or later
	jnb	aftfeb		;jmp if after Feb
	dec	ax		;decrement year
	add	ch,12		;Jan, Feb=months 13,14 of prev year
aftfeb: mov	bx,1461 	;=365.25*4
	mul	bx
	mov	bx,4
	div	bx
	push	ax		;component due to year
	mov	ax,153*256	;AH=153, AL=0
	xchg	al,ch		;AL=month, CX=day
	inc	ax
	mul	ah
	xor	dx,dx
	inc	bx		;=5
	div	bx		;AX=component due to month
	add	cx,ax		;add month component to day
	pop	ax
	add	cx,ax		;add year component
	mov	ax,cx
	xor	dx,dx
	mov	bl,7
	div	bx		;compute day of week in DX
	sub	cx,8829 	;normalize to 1/1/1980
	ret
todos	endp
;
;'fromdos' maps contiguous integer space into valid dates, from
;0 (=1/1/1980) thru 43888 (=2/28/2100). Transform is the inverse of
;that done by 'todos'
;
;in:	AX: number of days since 1/1/80
;out:	CH: month (1-12)
;	CL: day of month (1-31)
;	AX: year (1980-2100)
;	BX,DX: destroyed
;	SI,DI,BP,SS,DS,ES are preserved
;
fromdos proc	near
	add	ax,8829
	mov	cx,ax
	mov	bx,4
	mul	bx
	sub	ax,489		;adjust for desired bdy effects
	sbb	dx,0
	mov	bx,1461 	;=365.25*4
	div	bx
	push	ax		;AX=year origin 1956, -1 if Jan,Feb
	mul	bx
	mov	bx,4
	div	bx		;AX=integer component due to yr
	sub	cx,ax		;CX=day in year
	mov	ax,cx
	inc	bx		;=5
	mul	bx
	dec	ax
	mov	dl,153		;=30.6*5
	div	dl
	push	ax		;AL=month+1, with Jan,Feb = 13+1,14+1
	mul	dl
	xor	dx,dx
	div	bx		;AX=int component due to month
	sub	cx,ax		;CX=day in month
	pop	ax		;month+1
	dec	ax		;month
	mov	ch,al		;CH=month, CL=day
	cmp	al,13
	pop	ax		;year (origin 1956)
	jb	mthok		;jump if not Jan or Feb
	inc	ax		;Jan or Feb, adjust year up
	sub	ch,12		;..and month down
mthok:	add	ax,1956 	;1956 is origin year
	ret
fromdos endp
