/*
  DTF_USE.CPP
  This library allows one to access and display fonts made with
  dtfmaker and to use the default ROM BIOS font.  All of this must be done
  in mode 13h.

  Copyright (c) 1995.  All Rights Reserved.
*/

#include <mem.h>
#include "dtf_use.h"

DTFont::DTFont(char *fname)
{
  transparent = 1;
  font.s.space_string = NULL;
  font.character = NULL;
  font.palette.color = NULL;
  Load(fname);
}

DTFont::~DTFont()
{
  Delete();
}

void DTFont::Putch(int x, int y, int c, unsigned int color)
{
  long offset;
  char far *character;
  unsigned char bit_mask;
  unsigned long fh = font.h.height, fw = font.h.width;
  unsigned long character_size = (unsigned long)fh*(unsigned long)fw*(unsigned long)font.h.bits_per_pixel/8UL;
  unsigned long c_offset = (unsigned long)(((unsigned long)c)*character_size);
  unsigned int fc = color & 0x00FFU, fb = (color & 0xFF00U)>>8;

  character = ((char far *)font.character);

  offset = (y<<8)+(y<<6)+x;
  char cp = font.h.clear_pixel;
  if(font.h.bits_per_pixel == 1) {
    bit_mask = 0x80>>((fh*fw*font.h.bits_per_pixel*c)%8);
    for(int j = 0; j < fh; j++) {
      for(int i = 0; i < fw; i++) {
	if((*(character+c_offset) & bit_mask))
	  video_buffer[offset+i] = fc;
	else if(!transparent)
	  video_buffer[offset+i] = fb;

	if(bit_mask == 0x01) {
	  c_offset++;
	  bit_mask = 0x80;
	}
	else
	  bit_mask >>= 1;
      }
      offset += 320;
    }
  }
  else if(font.h.bits_per_pixel == 8) {
    for(int j = 0; j < fh; j++) {
      for(int i = 0; i < fw; i++) {
	if((color = *(character+c_offset)) != cp)
	  video_buffer[offset+i] = color;
	else if(!transparent)
	  video_buffer[offset+i] = cp;
	c_offset++;
      }
      offset += 320;
    }
  }
}

int DTFont::get_width(int c)
{
  int offset = -1;
  char far *character;
  unsigned char bit_mask;
  unsigned int fh = font.h.height, fw = font.h.width;
  unsigned long c_offset = (unsigned long)c*fh*fw*(unsigned long)font.h.bits_per_pixel/8UL;

  character = font.character;

  char cp = font.h.clear_pixel;
  if(font.h.bits_per_pixel == 1) {
    bit_mask = 0x80>>((fh*fw*font.h.bits_per_pixel*c)%8);
    for(int j = 0; j < fh; j++) {
      for(int i = 0; i < fw; i++) {
	if((*(character+c_offset) & bit_mask) && i > offset)
	  offset = i;

	if(bit_mask == 0x01) {
	  c_offset++;
	  bit_mask = 0x80;
	}
	else
	  bit_mask >>= 1;
      }
    }
  }
  else if(font.h.bits_per_pixel == 8) {
    for(int j = 0; j < fh; j++) {
      for(int i = 0; i < fw; i++) {
	if(((*(character+c_offset)) != cp) && (i > offset))
	  offset = i;
	c_offset++;
      }
    }
  }
  if(offset != -1)
    return offset+1;
  else
    return font.h.width/2+1;
}


void DTFont::Putstr(int x, int y, char far *str, unsigned int color)
{
  int i = 0;
  int offset, total_offset = 0;
  switch(font.s.space) {
    case Monospaced:  offset = font.h.width+1; break;
    case Mono_Script:  offset = font.h.width; break;
    case Script:  offset = 0; break;
    case Normal_Space:  offset = 0; break;
    default: offset = 0;
  }
  for(i = 0; str[i] != 0; i++) {
    switch(font.s.space) {
      case Script:  offset = get_width(str[i]); break;
      case Normal_Space:  offset = get_width(str[i])+1; break;
      case Font_Defined_Space:  offset = font.s.space_string[str[i]]; break;
    }
    Putch(x+total_offset, y, str[i], color);
    total_offset += offset;
  }
}

int DTFont::Load(char far *file_name, int bpp, unsigned int color)
{
  FILE far *fp;
  if(file_name != BIOS) {
    if(!(fp = fopen(file_name, "rb")))
      return 0;

    Delete();

    fread(&font.h, sizeof(DTF_Font_Header), 1, fp);
    fread(&font.s.space, sizeof(char), 1, fp);
    if(font.s.space == Font_Defined_Space) {
      font.s.space_string = (char far *)farmalloc(font.h.character_num);
      fread(font.s.space_string, sizeof(char), font.h.character_num, fp);
    }

    long font_size = ((unsigned int)font.h.height)*(unsigned int)font.h.width*(unsigned int)font.h.bits_per_pixel*(unsigned int)font.h.character_num/8U+1U;
    font.character = (char far *)farmalloc(font_size);
    fread(font.character, sizeof(char), font_size, fp);

    fread(&font.palette.color_num, sizeof(char), 1, fp);
    fread(&font.palette.color_offset, sizeof(char), 1, fp);
    font.palette.color = (RGB_color far *)farmalloc(font.palette.color_num*sizeof(RGB_color));
    fread(font.palette.color, sizeof(RGB_color), font.palette.color_num, fp);

    fclose(fp);
  }
  else {
    Delete();

    font.h.version = 1;
    font.h.height = 8;
    font.h.width = 8;
    font.h.bits_per_pixel = bpp;
    font.h.character_num = 256;
    font.h.clear_pixel = 0;
    font.s.space = Mono_Script;
    font.s.space_string = NULL;

    if(font.h.bits_per_pixel == 8) {
      unsigned long offset = 0, r_offset = 0;
      font.character = (char far *)farmalloc(16384); //256*8*8
      for(int i = 0; i < 128; i++) {
	for(int y = 0; y < 8; y++) {
	  for(int x = 0; x < 8; x++) {
	    if(*(rom_font+r_offset+y) & 0x80>>x)
	      *(font.character+offset+x) = color & 0x00FFU;
	    else
	      *(font.character+offset+x) = (color & 0xFF00U)>>8;
	  }
	  offset += 8;
	}
	r_offset += 8;
      }

      for(i = 8192; i < 16384; i++)
	*(font.character+i) = 0;
    }
    else {
      font.h.bits_per_pixel = 1;
      font.character = (char far *)farmalloc(2048); //256*8
      _fmemmove(font.character, rom_font, 1024);

      for(int i = 1024; i < 2048; i++)
	*(font.character+i) = 0;
    }

    font.palette.color_num = 0;
    font.palette.color_offset = 0;
    font.palette.color = NULL;
  }

  return 1;
}

int DTFont::Save(char far *file_name)
{
  FILE far *fp;
  if(!(fp = fopen(file_name, "wb")))
    return 0;

  fwrite(&font.h, sizeof(DTF_Font_Header), 1, fp);
  fwrite(&font.s.space, 1, 1, fp);
  if(font.s.space == Font_Defined_Space)
    fwrite(font.s.space_string, sizeof(char), font.h.character_num, fp);

  long font_size = ((unsigned int)font.h.height)*(unsigned int)font.h.width*(unsigned int)font.h.bits_per_pixel*(unsigned int)font.h.character_num/8U+1U;
  fwrite(font.character, sizeof(char), font_size, fp);

  fwrite(&font.palette.color_num, sizeof(char), 1, fp);
  fwrite(&font.palette.color_offset, sizeof(char), 1, fp);
  fwrite(font.palette.color, sizeof(RGB_color), font.palette.color_num, fp);

  fclose(fp);

  return 1;
}

void DTFont::Create(DTF_Font far *f)
{
  Delete();
  font.h.version = f->h.version;
  font.h.height = f->h.height;
  font.h.width = f->h.width;
  font.h.bits_per_pixel = f->h.bits_per_pixel;
  font.h.character_num = f->h.character_num;
  font.h.clear_pixel = f->h.clear_pixel;
  font.s.space = f->s.space;
  if(font.s.space == Font_Defined_Space) {
    font.s.space_string = (char far *)farmalloc(font.h.character_num);
    _fmemmove(font.s.space_string, f->s.space_string, font.h.character_num);
  }
  long font_size = ((unsigned int)font.h.height)*(unsigned int)font.h.width*(unsigned int)font.h.bits_per_pixel*(unsigned int)font.h.character_num/8U+1U;
  font.character = (char far *)farmalloc(font_size);
//  for(unsigned long l = 0; l < font_size; l++)
//    *(font.character+l) = *(f->character+l);
  _fmemmove(font.character, f->character, font_size-1);
  font.palette.color_num = f->palette.color_num;
  font.palette.color_offset = f->palette.color_offset;
  font.palette.color = (RGB_color far *)farmalloc(font.palette.color_num*sizeof(RGB_color));
  _fmemmove(font.palette.color, f->palette.color, font.palette.color_num*sizeof(RGB_color));
}

void DTFont::Retrieve(DTF_Font far *f)
{
  f->h.version = font.h.version;
  f->h.height = font.h.height;
  f->h.width = font.h.width;
  f->h.bits_per_pixel = font.h.bits_per_pixel;
  f->h.character_num = font.h.character_num;
  f->h.clear_pixel = font.h.clear_pixel;
  f->s.space = font.s.space;
  if(font.s.space == Font_Defined_Space) {
    f->s.space_string = (char far *)farmalloc(font.h.character_num);
    _fmemmove(f->s.space_string, font.s.space_string, font.h.character_num);
  }
  long font_size = ((unsigned int)font.h.height)*(unsigned int)font.h.width*(unsigned int)font.h.bits_per_pixel*(unsigned int)font.h.character_num/8U+1U;
  f->character = (char far *)farmalloc(font_size);
  _fmemmove(f->character, font.character, font_size);
  f->palette.color_num = font.palette.color_num;
  f->palette.color_offset = font.palette.color_offset;
  f->palette.color = (RGB_color far *)farmalloc(font.palette.color_num*sizeof(RGB_color));
  _fmemmove(f->palette.color, font.palette.color, font.palette.color_num*sizeof(RGB_color));

}

void DTFont::Delete()
{
  if(font.s.space_string != NULL)
    farfree(font.s.space_string);
  if(font.character != NULL)
    farfree(font.character);
  if(font.palette.color != NULL)
    farfree(font.palette.color);
}

void DTFont::Load_Palette()
{
  if(font.palette.color_num > 0)
    for(int i = 0; i < font.palette.color_num; i++)
      Set_Palette_Register(i+font.palette.color_offset, (font.palette.color+i));
}