/*
  DTF_USE.H
  Header file used to declare prototypes and structures for all functions
  in DTF_USE.CPP that allows one to access and display fonts made with
  dtfmaker and to use the default ROM BIOS font.  All of this must be done
  in mode 13h.

  Copyright (c) 1995.  All Rights Reserved.
*/

#ifndef __DTF_USE_H
#define __DTF_USE_H

#include <stdio.h>	//for fopen and fclose
#include <alloc.h>	//for farmalloc and farfree
#include "graph.h"

#define BIOS		NULL

enum Spacing_Options { Monospaced, Mono_Script, Script, Normal_Space,
			Font_Defined_Space };
//These are the supported spacing options, MONOSPACED places each character
//at full width with one pixel between each character, MONO_SCRIPT places
//each character at full width with no space between each character, SCRIPT
//places each character as wide as each character is in pixels with no
//space between each character (NOTE: having script as its spacing option
//doesn't necessarily mean that the font is a script font), NORMAL_SPACE
//places each character as wide as each character is in pixels with one
//space between each character, and FONT_DEFINED_SPACE places each character
//as wide as defined in the font, (definition for each character is a
//string of characters thats length is defined by the number of characters
//in the font, set to NULL and left out of file if space is not equal to
//FONT_DEFINED_SPACE) and adds an amount of space as defined by the space
//string if in excess of character width.

struct DTF_Font_Header {
  char version;		//Current version 1, used for backwards compatability
  char height, width;	//Height and Width of each character
  char bits_per_pixel;	//number of bits/pixel (1 to 8) dtf_maker 1.0 only supports 1-bit and 8-bit
  int  character_num;	//number of characters in font (generally 256)
  char clear_pixel; 	//color that is clear when written if transparent flag set (generally 0)
};

struct DTF_Font_Space {	//Included as a seperate header for easier coding for the load, save, create, and retrieve functions
  char space;		//space option chosen
  char far *space_string;	//space string used if Font_Defined_space set in space
};

struct DTF_Font_Trailer {
  char color_num;	//Number of colors in palette
  char color_offset;	//Starting register for palette placement
  RGB_color far *color;	//Palette for font if color_num > 0
};

struct DTF_Font {
  DTF_Font_Header h;
  DTF_Font_Space s;
  char far *character;
  DTF_Font_Trailer palette;
};

class DTFont {
  private:
    DTF_Font font;

    int transparent;

    void Delete();

    int get_width(int c);

  public:
    //Constructor and Destructor
    DTFont(char far *fname);
    ~DTFont();

    //Clear or Set Transparency flag
    void Set_Clear(int clear) { transparent = clear; };

    //Character and string writing functions that are the heart of this library
    void Putch(int x, int y, int c, unsigned int color = 15);
    void Putstr(int x, int y, char far *str, unsigned int color = 15);

    //This function allows for the user to build their own fonts or retrieve
    //the font from font above
    void Create(DTF_Font far *f);
    void Retrieve(DTF_Font far *f);

    //Initialize font and load it
    int Load(char far *file_name, int bpp = 1, unsigned int color = 15);
    int Save(char far *file_name);

    //Miscellaneous functions
    void Load_Palette();
    void Redirect_Output(char far *buffer)	//Used for double buffering
		{ video_buffer = buffer; };
};

#endif//__DTF_USE_H