//GRAPH.CPP

#include <io.h>
#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <bios.h>
#include <fcntl.h>
#include <memory.h>
#include <malloc.h>
#include <math.h>
#include <string.h>
#include "graph.h"

//Global Variables
unsigned char far *video_buffer = (char far *)0xA0000000L; // vram byte ptr
unsigned char far *rom_font = (char far *)0xF000FA6EL; // rom characters 8x8

//Special Variables

#define CURSOR_X_LEN	9
#define CURSOR_Y_LEN	11

char Cursor[] = {
  255, 255,  0,  0,  0,  0,  0,  0,  0,
  255, 15, 255,  0,  0,  0,  0,  0,  0,
  255, 15, 15, 255,  0,  0,  0,  0,  0,
  255, 15, 15, 15, 255,  0,  0,  0,  0,
  255, 15, 15, 15, 15, 255,  0,  0,  0,
  255, 15, 15, 15, 15, 15, 255,  0,  0,
  255, 15, 15, 15, 15, 15, 15, 255,  0,
  255, 15, 15, 15, 15, 15, 15, 15, 255,
  255, 15, 15, 255, 15, 15, 255, 255, 255,
  255, 255, 255, 255, 255, 15, 15, 255,  0,
   0,  0,  0,  0,  0, 255, 255, 255,  0
};

char Cursor2[] = {
  31, 31,  0,  0,  0,  0,  0,  0,  0,
  31, 15, 31,  0,  0,  0,  0,  0,  0,
  31, 15, 15, 31,  0,  0,  0,  0,  0,
  31, 15, 15, 15, 31,  0,  0,  0,  0,
  31, 15, 15, 15, 15, 31,  0,  0,  0,
  31, 15, 15, 15, 15, 15, 31,  0,  0,
  31, 15, 15, 15, 15, 15, 15, 31,  0,
  31, 15, 15, 15, 15, 15, 15, 15, 31,
  31, 15, 15, 31, 15, 15, 31, 31, 31,
  31, 31, 31, 31, 31, 15, 15, 31,  0,
   0,  0,  0,  0,  0, 31, 31, 31,  0
};


//FUNCTIONS/////////////////////////////////////////////////////////////////

//Palette Functions//
void Set_Palette_Register(int index, RGB_color_ptr color)
{
  outp(PALETTE_MASK,0xff);
  outp(PALETTE_REGISTER_WR, index);
  outp(PALETTE_DATA,color->red);
  outp(PALETTE_DATA,color->green);
  outp(PALETTE_DATA,color->blue);
}

void Set_Palette_Section(int start_index, int index_num, RGB_color_ptr colors)
{
  for(int i = start_index; i < start_index+index_num; i++)
    Set_Palette_Register(i, colors+(i*3));
}

void Get_Palette_Register(int index, RGB_color_ptr color)
{
  outp(PALETTE_MASK,0xff);
  outp(PALETTE_REGISTER_RD, index);
  color->red   = inp(PALETTE_DATA);
  color->green = inp(PALETTE_DATA);
  color->blue  = inp(PALETTE_DATA);
}

void Save_Palette(RGB_color_ptr pal)
{
  for(int i = 0; i < 256; i++)
    Get_Palette_Register(i, pal+(i*sizeof(RGB_color)));
}

void Restore_Palette(RGB_color_ptr pal)
{
  for(int i = 0; i < 256; i++)
    Set_Palette_Register(i, pal+(i*sizeof(RGB_color)));
}

//End Palette Functions//

//PCX Functions//

void PCX_Init(pcx_picture_ptr image)
{
  if(!(image->buffer = (char far *)_fmalloc(SCREEN_WIDTH * SCREEN_HEIGHT + 1))) {
    SetMode(0x3);
    printf("\nOh dear! We couldn't allocate the PCX screen buffer.");
    printf("\nCheck your memory with the mem command, and please");
    printf("\nnotify us if this error persists.");
    printf("\nYou have %u bytes left and occured in", farcoreleft());
    printf("\nfunction PCX_Init.");
    exit(1);
  }
}

void PCX_Delete(pcx_picture_ptr image)
{
  _ffree(image->buffer);
}

int PCX_Load(char *filename, pcx_picture_ptr image,int enable_palette)
{

  FILE *fp;
  int num_bytes,index;
  long count;
  unsigned char data;
  char far *temp_buffer;

  if(!(fp = fopen(filename,"rb")))
    return 0;

  temp_buffer = (char far *)image;

  for(index=0; index<128; index++)
    temp_buffer[index] = getc(fp);

  count=0;
  while(count<=SCREEN_WIDTH * SCREEN_HEIGHT) {
    data = getc(fp);
    if(data>=192) {
      num_bytes = data-192;
      data  = getc(fp);
      while(num_bytes-->0)
	image->buffer[count++] = data;
    }
    else
      image->buffer[count++] = data;
  }

  fseek(fp,-768L,SEEK_END);
  for(index=0; index<256; index++) {
    image->palette[index].red   = (getc(fp) >> 2);
    image->palette[index].green = (getc(fp) >> 2);
    image->palette[index].blue  = (getc(fp) >> 2);
  }

  fclose(fp);

  if(enable_palette)
    for(index=0; index<256; index++)
      Set_Palette_Register(index,(RGB_color_ptr)&image->palette[index]);

  return 1;
}

void PCX_Show_Buffer(pcx_picture_ptr image)
{
  char far *data;

  data = image->buffer;

  asm {
    push ds
    les di, video_buffer
    lds si, data
    mov cx,320*200/2
    cld
    rep movsw
    pop ds
  }
}

//End PCX Functions//

//Pixel Functions//

//End Pixel Functions//

//Double Buffer Functions//

//End Double Buffer Functions//

//Line Functions//

void HLine(int x1, int y, int x2, int color)
{
  if(y >= 0 && y < 200)
    for(int i = x1; i < x2+1; i++)
      if(i >= 0 && i < 320)
	video_buffer[(y<<8) + (y<<6) + i] = color;
}

void VLine(int x, int y1, int y2, int color)
{
  if(x >= 0 && x < 320)
    for(int i = y1; i < y2; i++)
      if(i >= 0 && i < 200)
	video_buffer[(i<<8) + (i<<6) + x] = color;
}

//End Line Functions//

//Mode Functions//

void SetMode(char mode)
{
  asm mov al,mode
  asm mov ah,0x00
  asm int 0x10
}

//End Mode Functions//

//Geometric Functions//

void Rectangle(int x1, int y1, int x2, int y2, int color)
{
  HLine(x1, y1, x2, color);
  HLine(x1, y2, x2, color);
  VLine(x1, y1, y2, color);
  VLine(x2, y1, y2, color);
}

void Bar(int x1, int y1, int x2, int y2, int color)
{
  for(int y = y1; y <= y2; y++)
    HLine(x1, y, x2, color);
}

//End Geometric Functions//

//Sprite Functions//

//End Sprite Functions//

//Special Effects Functions//

//End Special Effects Functions//

//Cursor Functions//

//Cursor Globals
char back_buf[CURSOR_X_LEN*CURSOR_Y_LEN];
int omx = -1, omy = -1;

//Cursor Globals to be modified by user
int c_height = 10, c_width = 10;
char *r_back_buf;

//Manual Cursor Buffer Functions
//Note:  These functions assume that the coordinates don't change between
//calls to the first and second, Put_Cursor_BBuf is called before
//Put_Cursor_Buf
//Puts Cursors Background Buffer
void Put_Cursor_BBuf(int mx, int my)
{
  if(omx != -1 && omy != -1) {
    for(int y = 0; y < CURSOR_Y_LEN; y++)
      for(int x = 0; x < CURSOR_X_LEN; x++)
	video_buffer[((omy+y)*320)+(omx+x)] = back_buf[(y*CURSOR_X_LEN)+x];
  }
  omx = mx;
  omy = my;
}

//Puts down cursor
void Put_Cursor_Buf(int mx, int my)
{
  for(int y = 0; y < CURSOR_Y_LEN; y++)
    for(int x = 0; x < CURSOR_X_LEN; x++) {
      back_buf[(y*CURSOR_X_LEN)+x] = video_buffer[((my+y)*320)+(mx+x)];
      if((Cursor[(y*CURSOR_X_LEN)+x] != 0) && (mx+x >= 0) &&
			(mx+x < 320) && (my+y >= 0) && (my+y <200))
	video_buffer[((my+y)*320)+(mx+x)] = Cursor[(y*CURSOR_X_LEN)+x];
    }
}

void Init_Rectangle_Cursor(int width, int height)
{
  c_height = height;
  c_width = width;
  r_back_buf = (char far *)farmalloc((width+2)*2+(height+2)*2-4);
  for(int i = 0; i < (width+2)*2+(height+2)*2-4; i++)
    r_back_buf[i] = 0;
  omx = -1; omy = -1;
}

void Rectangle_ABuffed(int mx, int my)
{
  if(omx != -1 && omy != -1) {
    for(int x = omx-1; x <= omx+c_width; x++) {
      if(x >= 0 && x < 320) {
	if(omy-1 >= 0 && omy-1 < 200)
	  video_buffer[x+((omy-1)<<6)+((omy-1)<<8)]               = r_back_buf[x-omx+1];
	if(omy+c_height >= 0 && omy+c_height < 200)
	  video_buffer[x+((omy+c_height)<<6)+((omy+c_height)<<8)] = r_back_buf[c_width+2+x-omx+1];
      }
    }
    for(int y = omy; y <= omy+c_height-1; y++) {
      if(y >= 0 && y < 200) {
	if(omx-1 >= 0 && omx-1 < 320)
	  video_buffer[omx-1+(y<<6)+(y<<8)]       = r_back_buf[(c_height+2)*2+(y-omy)*2];
	if(omx+c_width >= 0 && omx+c_width < 320)
	  video_buffer[omx+c_width+(y<<6)+(y<<8)] = r_back_buf[(c_height+2)*2+(y-omy)*2+1];
      }
    }
  }
  omx = mx;
  omy = my;

  for(int x = mx-1; x <= mx+c_width; x++) {
    if(x >= 0 && x < 320) {
      if(my-1 >= 0 && my-1 < 200)
	r_back_buf[x-mx+1] = video_buffer[x+((my-1)<<6)+((my-1)<<8)];
      if(my+c_height >= 0 && my+c_height < 200)
	r_back_buf[c_width+2+x-mx+1] = video_buffer[x+((my+c_height)<<6)+((my+c_height)<<8)];
    }
  }
  for(int y = my; y < my+c_height; y++) {
    if(y >= 0 && y < 200) {
      if(mx-1 >= 0 && mx-1 < 320)
	r_back_buf[(c_height+2)*2+(y-my)*2] = video_buffer[mx-1+(y<<6)+(y<<8)];
      if(mx+c_width >= 0 && mx+c_width < 320)
	r_back_buf[(c_height+2)*2+(y-my)*2+1] = video_buffer[mx+c_width+(y<<6)+(y<<8)];
    }
  }
  Rectangle(mx-1, my-1, mx+c_width, my+c_height, WHITE);
}

void Destroy_Rectangle_Cursor()
{
  farfree(r_back_buf);
}

//End Cursor Functions//