
// GRAPH1.H - header file for basic graphics and sprites

// D E F I N E S  ////////////////////////////////////////////////////////////

#ifndef __GRAPH_H
#define __GRAPH_H

#define ROM_CHAR_SET_SEG 0xF000  // segment of 8x8 ROM character set
#define ROM_CHAR_SET_OFF 0xFA6E  // begining offset of 8x8 ROM character set

#define VGA256            0x13
#define TEXT_MODE         0x03

#define PALETTE_MASK        0x3c6
#define PALETTE_REGISTER_RD 0x3c7
#define PALETTE_REGISTER_WR 0x3c8
#define PALETTE_DATA        0x3c9

#define SCREEN_WIDTH      (unsigned int)320
#define SCREEN_HEIGHT     (unsigned int)200

#define SPRITE_WIDTH	16
#define SPRITE_HEIGHT	16
#define SPRITES_PER_ROW	20

#define CHAR_WIDTH        8
#define CHAR_HEIGHT       8

#define BLACK		0
#define BLUE		1
#define GREEN		2
#define	CYAN		3
#define RED		4
#define MAGENTA		5
#define BROWN		6
#define LGRAY		7
#define DGRAY		8
#define LBLUE		9
#define LGREEN		10
#define LCYAN		11
#define	LRED		12
#define LMAGENTA	13
#define YELLOW		14
#define WHITE		15

// S T R U C T U R E S ///////////////////////////////////////////////////////

// this structure holds a RGB triple in three bytes

typedef struct RGB_color_typ
{
  unsigned char red;    // red   component of color 0-63
  unsigned char green;  // green component of color 0-63
  unsigned char blue;   // blue  component of color 0-63
} RGB_color, *RGB_color_ptr;

typedef struct pcx_header_typ
{
  char manufacturer;
  char version;
  char encoding;
  char bits_per_pixel;
  int x,y;
  int width,height;
  int horz_res;
  int vert_res;
  char ega_palette[48];
  char reserved;
  char num_color_planes;
  int bytes_per_line;
  int palette_type;
  char padding[58];
} pcx_header, *pcx_header_ptr;

typedef struct pcx_picture_typ
{
  pcx_header header;
  RGB_color palette[256];
  char far *buffer;
} pcx_picture, *pcx_picture_ptr;

//extern Set_Mode(int mode);
//extern void Draw_Ground_32();

extern unsigned char far *video_buffer;  // vram byte ptr
extern unsigned int far *video_buffer_w;  // vram word ptr
extern unsigned char far *rom_font;  // rom characters 8x8
extern unsigned char far *double_buffer;
extern unsigned int sprite_width;
extern unsigned int sprite_height;
extern int omx, omy;

//Palette Functions
void Set_Palette_Register(int index, RGB_color_ptr color);
void Get_Palette_Register(int index, RGB_color_ptr color);
void Set_Palette_Section(int start_index, int index_num, RGB_color_ptr colors);
void Clear_Palette();
void Save_Palette(RGB_color_ptr pal);
void Restore_Palette(RGB_color_ptr pal);

//Pixel Functions
void Plot_Pixel_Fast(int x, int y, unsigned char color);
void Plot_Pixel_Fast_D(int x, int y, unsigned char color);
unsigned char Get_Pixel(int x, int y);
unsigned char Get_Pixel_D(int x, int y);

//PCX Functions
void PCX_Init(pcx_picture_ptr image);
void PCX_Delete(pcx_picture *image);
int PCX_Load(char *filename, pcx_picture_ptr image,int enable_palette);
void PCX_Show_Buffer(pcx_picture_ptr image);
void PCX_Show_Buffer_D(pcx_picture_ptr image);

//Special Effects Functions
void Fade_In(pcx_picture_ptr image);
void Fade_Out();
void Set_Flame_Palette(RGB_color_ptr colors);
void Flame_In(pcx_picture_ptr image);
void Flame_Out();

//Double Buffer Functions
void Show_Double_Buffer();
void Init_Double_Buffer();

//Line Functions
void HLine(int x1, int y, int x2, int color);
void HLine_D(int x1, int y, int x2, int color);
void VLine(int x, int y1, int y2, int color);
void VLine_D(int x, int y1, int y2, int color);

//Mode Functions
void SetMode(char mode);

//Geometric Functions
void Rectangle(int x1, int y1, int x2, int y2, int color);
void Rectangle_D(int x1, int y1, int x2, int y2, int color);
void Bar(int x1, int y1, int x2, int y2, int color);	//Filled Rectangle
void Bar_D(int x1, int y1, int x2, int y2, int color);  //Filled Rectangle

//Sprite Functions
char *Grab_Sprite(pcx_picture_ptr image, int sprite_num, int w, int h);

//External Cursor Constants
extern int c_height, c_width;

//Cursor Functions
void Put_Cursor_ABuffed(int mx, int my);
void Put_Cursor_BBuf(int mx, int my);  //Manually places background buffer
void Put_Cursor_Buf(int mx, int my);   //Manually places cursor

void Init_Rectangle_Cursor(int height, int width);
void Rectangle_ABuffed(int mx, int my);
void Destroy_Rectangle_Cursor();

#endif//__GRAPH_H