;Copyright (c) Wally Bass, 1986, 1996.
;
	db	'(c)Wally Bass 1986,1996'
;
;'dayofwk' takes a Julian date in DX:AX and returns the day of week
;(0 thru 6 = Sun thru Sat) in DX. AX and BX are destroyed.
;
dayofwk proc	near
	mov	bx,7E07h
	div	bx
	xchg	ax,dx
	cwd
	xor	bh,bh
	inc	ax
	div	bx
	ret
dayofwk endp
;
;'divby4' divides DX:AX by 4. All other regs preserved
;
divby4	proc	near
	call	divby2
divby2: shr	dx,1
	rcr	ax,1
	ret
divby4	endp
;
;'times4' muliplies DX:AX by 4. All other regs preserved
;
times4	proc	near
	call	times2
times2: shl	ax,1
	rcl	dx,1
	ret
times4	endp
;
;'tojul' maps valid dates onto the contiguous integer space known as
;'Julian dates'
;
;in:	AX: calendar year (signed) (-4713 to -1, 1 to 32767)
;	CH: month (1 thru 12 = Jan thru Dec)
;	CL: day of month (1-31)
;	DI:SI transition date
;out:	DX:AX: Julian date
;	CX: destroyed
;	BX: destroyed
;	SI: destroyed
;	DI: destroyed
;	BP: preserved (unused)
;
tojul	proc	near
	test	ax,ax		;test sign of year
	push	si		;transition date low
	jns	toj02		;jump if positive
	inc	ax		;add 1 if negative
toj02:	add	ax,4900 	;add 4900 to year
	xchg	ax,si		;save adjusted year
	mov	ax,153*256	;AH=153, AL=0
	xchg	al,ch		;AL=month, CX=d
	cmp	al,3
	jnb	toj04		;jmp if after Feb
	dec	si		;adjust year down
	add	al,12		;adjust month up
toj04:	inc	ax		;AL=month+1
	mul	ah		;=30.6*5
	cwd
	mov	bx,5
	div	bx		;AX=component due to month
	add	cx,ax		;add month component to day
	mov	ax,1461 	;=365.25*4
	mul	si		;adjusted year
	call	divby4		;DX:AX=component due to year
	add	ax,cx		;add month/day to year component
	adc	dx,0
	sub	ax,0C7Ah	;68730=10C7Ah
	sbb	dx,1
	pop	cx		;transition date low
	push	dx
	push	ax
	xor	bx,bx
	sub	cx,ax
	sbb	di,dx
	jnb	toj06		;jump if Julian period
IF DO4K
	mov	ax,si		;adjusted year
else
	xchg	ax,si		;adjusted year to AX
endIF
	xor	dx,dx
	mov	bl,100
	div	bx		;AX=centuries since -4900
	mov	bl,3
	mul	bx
	inc	bx		;=4
	div	bx		;AX=lost leap years since -4900
	sub	ax,38
	xchg	ax,bx
IF DO4K
	xchg	ax,si		;adjusted year to AX
	sub	ax,4900
	xor	dx,dx
	mov	si,4000
	idiv	si
	add	bx,ax
endIF
toj06:	pop	ax
	pop	dx
	sub	ax,bx
	sbb	dx,0
	ret
tojul	endp
;
;'fromjul' maps Julian dates to calendar dates. Transform is the reverse
;of that done by 'tojul'
;
;in:	DX:AX: Julian date
;	DI:SI transition date
;out:	AL: month (1 thru 12 = Jan thru Dec)
;	CX: day of month (1-31)
;	SI: calendar year (signed)
;	AH: destroyed
;	BX: destroyed
;	DX: destroyed
;	DI: destroyed
;	BP: preserved (unused)
;
fromjul proc	near
	stc
	sbb	si,ax
	sbb	di,dx
	pushf
	add	ax,0C7Ah	;68730=10C7Ah
	adc	dx,1
	popf
	mov	bx,3
	jnb	fj08		;jump if Julian period
;Gregorian period
fj02	label	near
IF DO4K
	mov	si,0B066h	;set DI:SI= -(68730+1721120)
	mov	di,0FFE4h
	add	si,ax
	adc	di,dx
	jmp	short fj06
fj04:	add	ax,1
	adc	dx,0
fj06:	sub	si,4AE9h	;1460969=164AE9h
	sbb	di,16h
	jnb	fj04
endIF
	push	dx
	push	ax
	call	times4		;multiplying by 4
	sub	ax,643		;about ((30.6*4)+38)*4 (644 okay also)
	sbb	dx,0
	mov	si,48699	;days per 100 years=36524.25=48699*3/4
	div	si
	cwd
	div	bx		;AX=centuries since -4900
	mul	bx
	inc	bx
	div	bx		;AX=lost leap years since -4900
	sub	ax,38
	xchg	ax,cx
	pop	ax
	pop	dx
	add	ax,cx
	adc	dx,0
fj08:	push	ax		;adjusted Julian date, low
	call	times4		;multiply by 4
	sub	ax,489		;about 30.6*4*4
	sbb	dx,0
	mov	cx,1461 	;=365.25*4
	div	cx		;AX=year component
	mov	si,-4900
	add	si,ax		;year to SI
	mul	cx
	call	divby4		;DX:AX=integer component fm yr
	pop	dx		;Julian date, low
	sub	dx,ax		;DX=day in year, with Jan & Feb treated
				;following Dec of previous year
	xchg	ax,dx
	mov	cx,ax
	mov	bl,5
	mul	bx
	dec	ax
	mov	dl,153		;=30.6*5
	div	dl		;AL=month+1, with Jan,Feb = 13+1,14+1
	push	ax
	mul	dl
	cwd
	div	bx		;AX=int component due to month
	sub	cx,ax		;CX=day in month
	pop	ax		;month+1
	dec	ax		;month
	cmp	al,13
	jb	fj10		;jump if not Jan or Feb
	inc	si		;Jan or Feb, adjust year up
	sub	al,12		;..and month down
fj10:	test	si,si		;test for negative year or 0 year
	jg	fj12		;jump if >0
	dec	si		;adjust downward for B.C.
fj12	label	near
;;;;	cbw			;do this if you want the month in AX,
				;rather than just AL
	ret
fromjul endp
comment *
Some (perhaps) less obvious uses of 'tojul' and 'fromjul':

 1. Advance calendar 1 day:	year in AX, month in CH, day in CL
				call	tojul
				add	ax,1
				adc	dx,1
				call	fromjul

 2. Check if a date is valid:	year in AX, month in BH, day in BL
				push	ax
				push	cx
				call	tojul
				call	fromjul
				mov	ch,al
				pop	dx
				pop	ax
				cmp	ax,si
				jne	invalid_date
				cmp	cx,dx
				jne	invalid_date
*
