; This routine is part of VESA SVGA -library
;
; Copyright 1994 Johannes Lehtinen
; All rights reserved

model large,c
p386

include "jlvesads.asm"

extrn vesa_repos_w:far

segment jlvesa05_TEXT USE16 'CODE'
assume cs:jlvesa05_TEXT

; void JVRectangle_Draw(JVSWord x, JVSWord y, JVUWord width, JVUWord height,
;                       JVUByte color)
;
; Draws a block to screen with given color

; Some information

bl_linesleft   dw ?  ; Lines left to draw
bl_repolines   dw ?  ; Lines before next check
bl_filler      dd ?  ; Filler (color)

proc JVRectangle_Draw far
   public JVRectangle_Draw

   push  bp
   mov   bp,sp
   push  di
   push  ds
   push  es

   mov   ax,JLVesa_Data
   mov   ds,ax

   ; Make sure the block is on screen

   cmp   [word ptr ss:bp+6],0
   jge   short x_not_below

   ; X is less than 0

   mov   ax,[ss:bp+6]
   add   [ss:bp+10],ax
   mov   [word ptr ss:bp+6],0

x_not_below:
   cmp   [word ptr ss:bp+8],0
   jge   short y_not_below

   ; Y is less than 0

   mov   ax,[ss:bp+8]
   add   [ss:bp+12],ax
   mov   [word ptr ss:bp+8],0

y_not_below:
   mov   ax,[ss:bp+10]
   add   ax,[ss:bp+6]
   cmp   [ds:Width],ax
   jge   short width_not_above

   ; Wider than screen

   mov   ax,[ds:Width]
   sub   ax,[ss:bp+6]
   mov   [ss:bp+10],ax

width_not_above:
   mov   ax,[ss:bp+12]
   add   ax,[ss:bp+8]
   cmp   [ds:Height],ax
   jge   short height_not_above

   ; Heigher than screen

   mov   ax,[ds:Height]
   sub   ax,[ss:bp+8]
   mov   [ss:bp+12],ax

   ; Check that width or height is not negative

height_not_above:
   cmp   [word ptr ss:bp+10],0
   jle   end_of_block
   cmp   [word ptr ss:bp+12],0
   jle   end_of_block

   ; Make a filler to copy several bytes at once

   mov   al,[ss:bp+14]        ; AL = byte used as filler
   mov   [byte ptr cs:bl_filler],al
   shl   [cs:bl_filler],8
   mov   [byte ptr cs:bl_filler],al
   shl   [cs:bl_filler],8
   mov   [byte ptr cs:bl_filler],al
   shl   [cs:bl_filler],8
   mov   [byte ptr cs:bl_filler],al

   ; Calculate absolute address of starting point

   xor   eax,eax              ; Calculate address of start of line
   xor   ebx,ebx
   mov   ax,[ss:bp+8]
   mov   bx,[ds:LWidth]
   mul   ebx
   mov   bx,[ss:bp+6]         ; Calculate address of starting point
   add   eax,ebx
   add   eax,[ds:AStart]      ; Calculate address if scrolled
   mov   edx,eax

   ; Set linesleft internal variable

   mov   ax,[ss:bp+12]
   mov   [cs:bl_linesleft],ax

   ; Initialize registers used

   mov   ax,[ds:WWSeg]
   mov   es,ax
   cld                        ; Direction forward

   ; Reposition window if necessary

   cmp   [ds:WAStart],edx
   jbe   short not_bef_win
   call  far vesa_repos_w
   jmp   short go_drawing

not_bef_win:
   mov   eax,[ds:WAStart]
   add   eax,[ds:WSize]
   cmp   edx,eax
   jb    short go_drawing
   call  far vesa_repos_w

   ; Calculate current offset

go_drawing:
   mov   edi,edx
   sub   edi,[ds:WAStart]

   ; Drawing loop

   ; First calculate, how many lines can be drawn without address overflow

draw_loop:
   mov   eax,[ds:WAStart]
   add   eax,[ds:WSize]
   sub   eax,edx
   xor   ebx,ebx
   mov   bx,[ds:LWidth]
   push  edx
   xor   edx,edx
   div   ebx
   pop   edx
   cmp   ax,0
   je    short special_line

   ; Draw lines

   cmp   [cs:bl_linesleft],ax ; Check if fewer lines left than possible to draw on the page
   jae   short jump_0
   mov   ax,[cs:bl_linesleft]

jump_0:
   mov   [cs:bl_repolines],ax
   sub   [cs:bl_linesleft],ax
   mov   eax,[cs:bl_filler]   ; Register containing filler

draw_line:
   mov   cx,[ss:bp+10]        ; Width of the block
   cmp   cx,3
   jb    short jump_3

   test  di,1                 ; Check if need to write one byte
   jz    short jump_1
   stosb
   dec   cx

jump_1:
   test  di,2                 ; Check if need to write two bytes
   jz    short jump_2
   stosw
   sub   cx,2

jump_2:
   mov   bx,cx                ; Write four bytes at time
   shr   cx,2
   jz    short jump_3

   rep   stosd
   mov   cx,bx

jump_3:
   test  cx,2
   jz    short jump_4
   stosw

jump_4:
   test  cx,1
   jz    short jump_5
   stosb

jump_5:
   add   di,[ds:LWidth]       ; Move to the start of next line
   sub   di,[ss:bp+10]

   xor   ecx,ecx
   mov   cx,[ds:LWidth]
   add   edx,ecx
   dec   [cs:bl_repolines]
   jnz   draw_line

   ; Draw line, which is between two pages

special_line:
   cmp   [cs:bl_linesleft],0
   je    end_of_block
   mov   eax,[cs:bl_filler]   ; EAX = Filler

   mov   ecx,[ds:WAStart]     ; Calculate how line is divided
   add   ecx,[ds:WSize]
   sub   ecx,edx
   cmp   [ss:bp+10],cx
   jbe   short line_on_current   ; Line on current page

   ; Line is divided on both pages

   mov   bx,[ss:bp+10]        ; BX = length of the line on next page
   sub   bx,cx
   rep   stosb                ; Draw line on current page

   xor   ecx,ecx
   mov   cx,[ds:LWidth]
   add   edx,ecx              ; Switch on next page
   push  bx
   call  far vesa_repos_w
   pop   bx
   mov   eax,[cs:bl_filler]
   mov   edi,edx              ; Calculate new DI
   xor   ecx,ecx
   mov   cx,[ds:LWidth]
   sub   edi,ecx
   xor   ecx,ecx
   mov   cx,[ss:bp+10]
   add   edi,ecx
   sub   edi,[ds:WAStart]
   sub   di,bx

   mov   cx,bx                ; Draw line on next page
   rep   stosb
   xor   ecx,ecx
   add   di,[ds:LWidth]       ; Move on the next line
   sub   di,[ss:bp+10]
   dec   [cs:bl_linesleft]
   jnz   draw_loop
   jmp   short end_of_block

   ; Line is on current page

line_on_current:
   mov   cx,[ss:bp+10]        ; Draw line on this page
   mov   eax,[cs:bl_filler]
   rep   stosb

   xor   ecx,ecx
   mov   cx,[ds:LWidth]
   add   edx,ecx
   call  far vesa_repos_w
   mov   edi,edx              ; Calculate new DI
   sub   edi,[ds:WAStart]
   dec   [cs:bl_linesleft]
   jnz   draw_loop

end_of_block:
   pop   es
   pop   ds
   pop   di
   pop   bp
   retf

endp JVRectangle_Draw

ends

end
