*  BOF: TIMEGET.CH

/*
 * File......: TIMEGET.CH
 * Author....: Berend Tober
 * CIS ID....: 70541,1030
 * Date......: Thu  95/04/20 14:09:45
 */

/* Modification History
*/


/***
*  @..GET
*
*  This is a modified version of the @..GET in CA's std.ch file.  The
*  difference is in when the :display() message is sent to the newly-
*  created get object.  Here it is sent AFTER the optional message <msg>
*  is sent to the object, so that the display incorporates the new block.
*/

#command @ <row>, <col> GET <var>                                          ;
                        [PICTURE <pic>]                                    ;
                        [VALID <valid>]                                    ;
                        [WHEN <when>]                                      ;
                        [SEND <msg>]                                       ;
                                                                           ;
      => SetPos( <row>, <col> )                                            ;
       ; AAdd( GetList, _GET_( <var>, <"var">, <pic>, <{valid}>, <{when}>));
       [; ATail(GetList):<msg>]                                            ;
       ; atail( getList ):display()


/***
*  @..GET..TIMEVALUE
*
*  This is the new preprocessor command that does formatted time-value gets
*/

#command @ <row>, <col> GET <var>                                       ;
                        TIMEVALUE                                       ;
                        [<clauses,...>]                                 ;
                                                                        ;
      => @ <row>, <col> GET <var>                                       ;
        [<clauses>]                                                     ;
        SEND block := {|x| secToMil(iif( x==nil, <var>, <var> := milToSec(x)))}


/*
 *  $DOC$
 *  $FUNCNAME$
 *     @..GET..TIMEVALUE
 *  $CATEGORY$
 *     INTERFACE
 *  $ONELINER$
 *     Creates a get object for reading formatted time values.
 *  $SYNTAX$
 *  @ <row>, <col> GET <var> TIMEVALUE [<clauses,...>]
 *  $ARGUMENTS$
 *  <row> is the screen row where the GET is to appear.
 *
 *  <col> is the screen column where the GET is to appear.
 *
 *  <var> is the get varialble of numeric type representing time as
 *  the number of seconds past midnight.
 *
 *  TIMEVALUE is a required keyword that identifies this get as
 *  a formatted time value get.
 *
 *  [<clauses,...>] are any other clauses typically associated with
 *  the Clipper @ ... GET command, in particular, the PICTURE clause
 *  is useful with any of the following the format strings:
 *
 *     "@R 9999"         Military time format.
 *     "@R 99:99"        24-hour clock time format.
 *     "@R 99:99:99"     Hours/minutes/seconds format.
 *     "@R 99:99:99.hh"  Hours/minutes/seconds/hundredths format.
 *
 *  $RETURNS$
 *     Adds a get object to the currently-visible getList array.
 *  $DESCRIPTION$
 *     This is a special-purpose @ ... GET command that sets up a
 *     get object with a display block (oGet:block) defined so as to
 *     format numeric time value variables as time strings.
 *
 *     Its use requires including the header file "timeget.ch" as well
 *     as linking in the module timeget.obj.  The header file defines
 *     a pre-processer command that creates a new get object and adds
 *     it the to the currently in-scope getList array in a fashion
 *     similar to the standard Clipper @ ... GET command.
 *
 *     The important difference is in how the get object display block
 *     is defined.  The block is defined so as to format numeric values
 *     representing the number of seconds past midnight as time
 *     strings.
 *  $EXAMPLES$
 *     #include "timeget.ch"
 *     /*
 *      * <more program statements>
 *      */
 *
 *     @ 5, 0 SAY 'Number  ' GET cCustNmbr
 *     @ 6, 0 SAY 'Company ' GET cCompany
 *     @ 7, 0 SAY 'Date    ' GET dLastDay
 *     @ 8, 0 SAY 'Time    ' GET nLastTime TIMEVALUE
 *                                        // Note:  0 <= nLastTime < 86,400
 *                                        // represents number of seconds
 *     READ                               // past midnight.
 *
 *  $SEEALSO$
 *  $INCLUDE$
 *     timeget.ch
 *  $END$
 */

* EOF: TIMEGET.CH

