*  BOF: TIMEGET.PRG

/*
 * File......: TIMEGET.PRG
 * Author....: Berend Tober
 * CIS ID....: 70541,1030
 * Date......: Thu  95/04/20 14:07:22
 */

/* Modification History
*/

********************************** milToSec() **********************************
function milToSec( cMilitaryTime )
   local cTime    := strtran(strtran( cMilitaryTime, ":",""),".","")+"00000000"
   local nHour    := val(substr( cTime, 1, 2))
   local nMin     := val(substr( cTime, 3, 2))
   local nSec     := val(substr( cTime, 5, 2))
   local nHund    := val(substr( cTime, 7, 2))
   local nTime    := (nHour*3600 + nMin*60 + nSec + nHund/100)%86400
   return nTime
*  end of milToSec()

********************************** secToMil() **********************************
function secToMil( nSeconds )
   local nTime    := if(nSeconds == nil, seconds(), nSeconds)%86400
   local nHour    := int(nTime/3600)
   local nMin     := int((nTime - (3600*nHour))/60)
   local nSec     := int(nTime - 60*(nMin+(60*nHour)))
   local nHund    := 100*(nTime - int( nTime ))
   local bPad     := {|x| right("00"+alltrim(str(x,2)),2)}
   local cTime    := ;
      eval( bPad, nHour )+;
      eval( bPad, nMin  )+;
      eval( bPad, nSec  )+;
      eval( bPad, nHund )

   return cTime
*  end of secToMil()

/*
 *  $DOC$
 *  $FUNCNAME$
 *     milToSec()
 *  $CATEGORY$
 *     DATE/TIME
 *  $ONELINER$
 *     Converts militart time strings to time values
 *  $SYNTAX$
 *     milToSec( <cMilitaryTime> ) --> nTime
 *  $ARGUMENTS$
 *     <cMilitaryTime> is a time string in military format, i.e.,
 *     HHMMSShh.  The string may be truncated from the right,
 *     omitting the least significant values.
 *  $RETURNS$
 *     A Clipper numeric representing the number of secons past
 *     midnight equivalent to <cMilitaryTime>
 *  $DESCRIPTION$
 *     This function converts time strings in military format to time
 *     values measured in seconds past midnight.   The string may
 *     contain the normal time delimiters ":" and ".", i.e., formatted
 *     as HH:MM:SS.hh, and may be truncated from the right, i.e., HH:MM.
 *  $EXAMPLES$
 *  $SEEALSO$
 *  $INCLUDE$
 *  $END$
 */

/*
 *  $DOC$
 *  $FUNCNAME$
 *     secToMil()
 *  $CATEGORY$
 *     DATE/TIME
 *  $ONELINER$
 *     Converts Clipper time values into time strings
 *  $SYNTAX$
 *     secToMil( [<nSeconds>] ) -> cMilitaryTime
 *  $ARGUMENTS$
 *     <nSeconds> is a Clipper numeric variable or expression of
 *     time as measured in number of seconds past midnight.
 *     Defaults to seconds()
 *  $RETURNS$
 *     A string representing time in military format: HHMMSShh
 *     where HH is hours, MM is minutes, SS is seconds, and hh is
 *     hundredths of seconds.
 *  $DESCRIPTION$
 *     This function converts a number representing time in seconds
 *     past midnight to a string representing time in 24-hour, military
 *     time format.  The returned string is eight characters long with
 *     no punctuation and with two characters for each of the following
 *     values: hours, minutes, seconds, hundredths of seconds.
 *  $EXAMPLES$
 *  $SEEALSO$
 *  $INCLUDE$
 *  $END$
 */

* EOF: TIMEGET.PRG

